<?php

function getsoapy_settings_init()
{

    // register a new section in the "reading" page
    add_settings_section(
        'getsoapy_settings_section',
        'getSoapy Settings',
        'getsoapy_settings_section_callback',
        'getsoapy'
    );

    add_settings_section(
        'getsoapy_settings_advanced',
        'Advanced Settings',
        'getsoapy_settings_advanced_callback',
        'getsoapy'
    );

    $fields = [
        'getsoapy_settings_section' => [
            // ['name', 'label', 'callback']
            ['getsoapy_business_id', 'Business ID', 'getsoapy_settings_bid_field_callback', 'string', 'sanitize_text_field'],
            ['getsoapy_hide_fields', 'Hide fields', 'getsoapy_settings_hf_field_callback', 'string', 'sanitize_text_field'],
            ['getsoapy_styles', 'Override default styles', 'getsoapy_settings_styles_field_callback', 'string', 'sanitize_text_field'],
            ['getsoapy_show_fab', 'Floating quote button', 'getsoapy_settings_show_fab', 'boolean'],
            ['getsoapy_quote_in_pages', 'Paginate quotes', 'getsoapy_settings_quote_in_pages', 'boolean'],
        ],
        'getsoapy_settings_advanced' => [
            ['getsoapy_disable_styles', 'Disable stylesheet', 'getsoapy_settings_disable_styles_field_callback', 'boolean', null],
            ['getsoapy_plugin_options', 'Plugin options', 'getsoapy_settings_plugin_options_field_callback', 'string', null],
        ]
    ];

    foreach ($fields as $section => $_fields) {
        foreach ($_fields as $field) {

            register_setting('getsoapy', $field[0], [
                'type' => $field[3],
                'sanitize_callback' => $field[4]
            ]);

            add_settings_field(
                $field[0],
                $field[1],
                $field[2],
                'getsoapy',
                $section
            );
        }
    }
}

/**
 * register getsoapy_settings_init to the admin_init action hook
 */
add_action('admin_init', 'getsoapy_settings_init');

function getsoapy_options_page()
{
    add_plugins_page(
        'getSoapy',
        'getSoapy',
        'manage_options',
        'getsoapy',
        'getsoapy_options_page_html'
    );
}

function getsoapy_options_page_html()
{
    // check user capabilities
    if (!current_user_can('manage_options')) {
        return;
    }

    // add error/update messages

    // check if the user have submitted the settings
    // WordPress will add the "settings-updated" $_GET parameter to the url
    if (isset($_GET['settings-updated'])) {
        // add settings saved message with the class of "updated"
        flush_rewrite_rules();
        add_settings_error('getsoapy_messages', 'getsoapy_message', __('Settings Saved', 'getsoapy'), 'updated');
    }

    // show error/update messages
    settings_errors('getsoapy_messages');
?>
    <div class="wrap">
        <h1><?php echo esc_html(get_admin_page_title()); ?></h1>
        <form action="options.php" method="post">
            <?php
            // output security fields for the registered setting "getsoapy"
            settings_fields('getsoapy');
            // output setting sections and their fields
            // (sections are registered for "getsoapy", each field is registered to a specific section)
            do_settings_sections('getsoapy');
            // output save settings button
            submit_button('Save Settings');
            ?>
        </form>
    </div>
<?php
}


/**
 * Register our getsoapy_options_page to the admin_menu action hook.
 */
add_action('admin_menu', 'getsoapy_options_page');

/**
 * callback functions
 */

function getsoapy_settings_section_callback()
{
    echo '<p class="description">You can <a href="https://getsoapy.co.uk/docs/wordpress-plugin" target="_blank">read the documentation on these options</a>.</p>';
}

function getsoapy_settings_advanced_callback()
{
    echo '<p class="description">You probably don\'t need to worry about these settings.</p>';
}

function getsoapy_settings_bid_field_callback()
{
    $html = getsoapy_settings_field_callback('getsoapy_business_id');
    $businessId = get_option('getsoapy_business_id', '');

    $url = get_site_url();
    // $url = 'scotclean.co.uk';
    $html .= "<script type=\"text/javascript\">document.addEventListener('DOMContentLoaded', function() {
        fetch('https://getsoapy.com/api/embed/suggestBusinessId?url={$url}')
            .then(response => response.json())
            .then(data => {
                let message;
                if (data.success && data.data.businessId === '{$businessId}') {
                    message = 'Business ID matches business \''+data.data.name+'\'';
                } else if (data.success !== true) {
                    message = 'Unable to find a website registered for \'{$url}\'. <br />Please register at <a href=\"https://app.getsoapy.com\">app.getsoapy.co.uk</a> and set your website address.';
                } else {
                    message = 'Look like the businessID for \''+data.data.name+'\' is '+data.data.businessId+'<br />';
                    message += '<button class=\"button\" onClick=\"document.getElementById(\'getsoapy_business_id\').value = \''+data.data.businessId+'\'\">Set Business ID to '+data.data.businessId+'</button>'
                }

                document.getElementById('getsoapy_bid_suggestion').innerHTML = message;
            });
    });</script>";
    $html .= '<p id="getsoapy_bid_suggestion"></p>';

    echo $html;
}

function getsoapy_settings_hf_field_callback()
{
    echo getsoapy_settings_field_callback(
        'getsoapy_hide_fields',
        "List fields you don't want to show customers when requesting a quote. Leave blank to show all fields."
    );
}

function getsoapy_settings_show_fab()
{
    echo getsoapy_settings_checkbox_callback(
        'getsoapy_show_fab',
        'Show a floating "Request a quote" button in the bottom right of any page without a quote form.',
        true
    );
}

function getsoapy_settings_quote_in_pages()
{
    echo getsoapy_settings_checkbox_callback(
        'getsoapy_quote_in_pages',
        'Splitting the form into pages can increases the likelihood of customers completing requests.',
        true
    );
}

function getsoapy_settings_styles_field_callback()
{
    echo getsoapy_settings_field_callback(
        'getsoapy_styles',
        'See the documentation linked above for the options you can list here.'
    );
}

function getsoapy_settings_disable_styles_field_callback()
{
    echo getsoapy_settings_checkbox_callback(
        'getsoapy_disable_styles',
        "Completely disable loading of all styles for the widgets. This gives you complete control over their appearance and negates the 'default styles' option.<br />
        <strong>Warning: You'll need style all aspects of the plugin, including the 'Floating Quote Button'.</strong>",
        false
    );
}

function getsoapy_settings_plugin_options_field_callback()
{
    echo getsoapy_settings_json_callback(
        'getsoapy_plugin_options',
        'Change options in the JavaScript plugin which are not available above.<br />
        Provide a JSON object with the options you\'d like to change.'
    );
}

function getsoapy_settings_checkbox_callback($field, $help, $default)
{
    $html = '<input type="checkbox" name="' . $field . '" id="' . $field . '" value="1" ' . checked(1, get_option($field, $default), false) . ' />';

    if ($help) {
        $html .= '<p class="description"><label for="' . $field . '">' . $help . '</label></p>';
    }

    return $html;
}

function getsoapy_settings_field_callback($field, $help = null)
{
    $setting = get_option($field);

    $html = '<input type="text" name="' . $field . '" id="' . $field . '" value="' . (isset($setting) ? esc_attr($setting) : '') . '" class="regular-text ltr">';

    if ($help) {
        $html .= '<p class="description">' . $help . '</p>';
    }

    return $html;
}


function getsoapy_settings_json_callback($field, $help = null)
{
    $setting = get_option($field, '{}');

    $html = '<textarea name="' . $field . '" id="' . $field . '" class="regular-text ltr">' . $setting . '</textarea>';

    if ($help) {
        $html .= '<p class="description">' . $help . '</p>';
    }

    json_decode($setting);
    if ($setting !== '' && json_last_error()) {
        $error = json_last_error_msg();
        $html .= '<p class="notice notice-error">Invalid JSON. These options will be ignored.<br />Error: ' . $error . '</p>';
    }

    return $html;
}
